% Copyright (C) 2001, James B. Rawlings and John G. Ekerdt
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2, or (at
% your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
% MA 02111-1307, USA.
%
% Revised 8/13/2018

%
% limit cycle parameters
%
% units are kmol, min, kJ, K, m^3
%

p = struct();
p.k_m      = 0.004;
p.T_m      = 298;
p.E        = 15000;
p.c_Af     = 2;
p.C_p      = 4;
p.rho      = 1000;
p.C_ps     = p.C_p*p.rho;
p.T_f      = 298;
p.T_a      = p.T_f;
p.DeltaH_R = -2.2e5;
p.U        = 340;
p.theta    = 35;
p.T_set    = 321.53;
p.c_set    = 0.48995;
p.T_fs     = p.T_f;
p.Kc       = 0;
p.gamma    = p.E/p.T_f;
p.B        = -p.DeltaH_R*p.c_Af*p.gamma/(p.C_ps*p.T_f);
p.beta     = p.U/p.C_ps*p.theta;
p.Da       = p.k_m*exp(-p.E*(1/p.T_f-1/p.T_m))*p.theta;
p.x2c      = (p.T_a-p.T_f)/p.T_f*p.gamma;


%
% compute 5 dynamic trajectories to show phase portrait
%

%x0 = [p.c_set; p.T_set];
x0 = [p.c_Af; p.T_f];
tfinal = 5*p.theta;
ntimes = 200;
tout   = linspace(0, tfinal, ntimes);
opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, x] = ode15s(@(t, x) rhs(t, x, p), tout, x0, opts);
u = (x(:,2) - p.T_set)*p.Kc + p.T_fs;
conv = (p.c_Af - x(:,1))/p.c_Af;
table = [conv x(:,2)];

x0=[0; p.T_f];
tfinal = 5*p.theta;
ntimes = 200;
tout   = linspace(0, tfinal, ntimes);
opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, x] = ode15s(@(t, x) rhs(t, x, p), tout, x0, opts);
u = (x(:,2) - p.T_set)*p.Kc + p.T_fs;
conv = (p.c_Af - x(:,1))/p.c_Af;
table = [table conv x(:,2)];

x0=[0.3980; 321.39];   %x=0.8010, the unstable steady state IC
tfinal = 5*p.theta;
ntimes = 200;
tout   = linspace(0, tfinal, ntimes);
opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, x] = ode15s(@(t, x) rhs(t, x, p), tout, x0, opts);
u = (x(:,2) - p.T_set)*p.Kc + p.T_fs;
conv = (p.c_Af - x(:,1))/p.c_Af;
table = [table conv x(:,2)];

x0=[.8, 310];
tfinal = 5*p.theta;
ntimes = 200;
tout   = linspace(0, tfinal, ntimes);
opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, x] = ode15s(@(t, x) rhs(t, x, p), tout, x0, opts);
u = (x(:,2) - p.T_set)*p.Kc + p.T_fs;
conv = (p.c_Af - x(:,1))/p.c_Af;
table = [table conv x(:,2)];

x0=[.6, 315];
tfinal = 5*p.theta;
ntimes = 200;
tout   = linspace(0, tfinal, ntimes);
opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, x] = ode15s(@(t, x) rhs(t, x, p), tout, x0, opts);
u = (x(:,2) - p.T_set)*p.Kc + p.T_fs;
conv = (p.c_Af - x(:,1))/p.c_Af;
table = [table conv x(:,2)];

save -ascii phase_portrait.dat table;

if (~ strcmp (getenv ('OMIT_PLOTS'), 'true')) % PLOTTING
    plot (table(:,2),table(:,1),table(:,4),table(:,3),...
             table(:,6),table(:,5),table(:,8),table(:,7),...
             table(:,10),table(:,9));
    % TITLE
end % PLOTTING