function [resid, rate] = pelletfull(x, p)
  za = x(1:p.npts);
  zb = x(p.npts+1:2*p.npts);
  zc = x(2*p.npts+1:3*p.npts);
  ca = exp(za);
  cb = exp(zb);
  cc = exp(zc);

  k1      = p.k10*exp(-p.E1/p.T);
  k2      = p.k20*exp(-p.E2/p.T);
  Ka      = p.Ka0*exp(-p.Ea/p.T);
  Kc      = p.Kc0*exp(-p.Ec/p.T);
  den     = (1+Ka*ca+Kc*cc).^2;
  r1      = k1.*ca.*cb./den;
  r2      = k2*cc.*cb./den;
  Ra      = - r1;
  Rb      = - 1/2*r1  - 9/2*r2;
  Rc      = - r2;
  % component A
  ip = 1;
  resid(ip:ip+p.npts-1) = p.B*za + p.A*za.*(p.A*za + 2./p.R) + Ra./(p.Da*ca);
  resid(ip) = p.A(1,:)*za;
  resid(ip+p.npts-1) = p.Da*p.A(end,:)*za - p.kma*(p.caf/ca(end) - 1);

  % component B
  ip = p.npts+1;
  resid(ip:ip+p.npts-1) = p.B*zb + p.A*zb.*(p.A*zb + 2./p.R) + Rb./(p.Db*cb);
  resid(ip) = p.A(1,:)*zb;
  resid(ip+p.npts-1) =  p.Db*p.A(end,:)*zb - p.kmb*(p.cbf/cb(end) - 1);

  % component C
  ip = 2*p.npts+1;
  resid(ip:ip+p.npts-1) = p.B*zc + p.A*zc.*(p.A*zc + 2./p.R) + Rc./(p.Dc*cc);
  resid(ip)  = p.A(1,:)*zc;
  resid(ip+p.npts-1) = p.Dc*p.A(end,:)*zc - p.kmc*(p.ccf/cc(end) - 1);

  % pellet surface derivatives (for production rates in bed function)
  rate = [p.A(end,:)*ca;
	  p.A(end,:)*cb;
	  p.A(end,:)*cc];